# encoding: utf-8
# Created by chenghaomou at 2019-05-01
import re
from dataclasses import dataclass
import kenlm
from Patcher.Retriever import Retriever
from Patcher.Tokenizer import Tokenizer


@dataclass
class Editor:

    tokenizer: Tokenizer
    retriever: Retriever
    lm: kenlm.Model
    eng_vocab: set

    @staticmethod
    def edit_token(prev: list, token: str, trans: list) -> list:

        new_trans = []
        for t in prev:
            for tran in trans:
                new_trans.append(t.replace(token, tran))
        return new_trans

    def edit_sentence(self, source: str, target: str, online: False) -> tuple:

        OOV = set()

        tokens = self.tokenizer.tokenize(target)
        trans = [target]

        search = self.retriever.search if not online else self.retriever.search_online

        for token in tokens:
            if str(token).isascii():
                if token in source:
                    if bool(re.match(r'^\'?[a-zA-Z]+[-\']?[a-zA-Z]+$', token)) \
                            and token not in self.eng_vocab \
                            and token.lower() not in self.eng_vocab:
                        OOV.add(token)
                        trans = self.edit_token(trans, token, search(token))
                        continue
                    else:
                        continue
            else:
                OOV.add(token)
                trans = self.edit_token(trans, token, search(token))

        return self.best_translation(trans), OOV

    def best_translation(self, candidates):

        return list(sorted(candidates, key=self.lm.score, reverse=True))[0]


if __name__ == "__main__":

    from Patcher.Utils import Romanizer, load_dictionary, load_vocabulary
    import spacy
    from google.cloud import translate

    LM_PATH = "/Users/chenghaomou/Documents/Code-ProjectsPyCharm/Elisa/lm/elisa.arpa.en"
    ROMANIZATION = "/Users/chenghaomou/Documents/Code-ProjectsPyCharm/dict-patch/uroman"
    ENGLISH_VOCAB = "/Users/chenghaomou/Documents/Code-ProjectsPyCharm/dict-patch/300k.txt"
    ZUL_DICT = "/Users/chenghaomou/Documents/Code-ProjectsPyCharm/dict-patch/zul.lexicon.norm"
    FLAT = '/Users/chenghaomou/Documents/Code-ProjectsPyCharm/dict-patch/isi-sbmt.zul-eng.dev.y3r1.v3.flat'

    lm = kenlm.Model(LM_PATH)
    dictionary = load_dictionary(ZUL_DICT)
    eng_vocab = load_vocabulary(ENGLISH_VOCAB)
    romanizer = Romanizer(ROMANIZATION,
                          language_code='zul')
    tokenizer = Tokenizer(basic_tokenizer=spacy.load('en_core_web_lg'),
                          romanizer=romanizer)
    retriever = Retriever(candidates=dictionary,
                          language='zul',
                          translate_client=translate.Client(),
                          romanizer=romanizer,
                          lowercase=True,
                          top_k=5)
    editor = Editor(eng_vocab=eng_vocab, tokenizer=tokenizer, retriever=retriever, lm=lm)

    for (source, target) in map(lambda l: l.strip('\n').split('\t'), open(FLAT).readlines()):
        res = editor.edit_sentence(source, target, online=True)
        print(res)

        res = editor.edit_sentence(source, target, online=False)
        print(res)

        print('*'*100)


