# encoding: utf-8
# Created by chenghaomou at 2019-05-01
import re
from dataclasses import dataclass

from Patcher.Utils import Romanizer


@dataclass
class Tokenizer:

    basic_tokenizer: callable
    romanizer: Romanizer

    def tokenize(self, sentence: str) -> list:
        tokens = []
        raw_tokens = sentence.strip('\n').split(' ')
        romanized_raw_tokens = self.romanizer.run(sentence.strip('\n')).split(' ')
        for raw_token, fine_tokens in zip(raw_tokens, self.basic_tokenizer(romanized_raw_tokens)):
            # This token need no further tokenization
            if fine_tokens and raw_token.startswith(fine_tokens[0]) and len(fine_tokens) == 1:
                tokens += fine_tokens
            elif fine_tokens:
                # Tokenize it one more time if it
                # has more than one word after romanization;
                # Or has symbols and letters in it
                if len(fine_tokens) > 1 or any(str(c).isalpha() for c in fine_tokens[0]):
                    raw_token = re.sub(r'([,<>\.\/\?;:\'\"\[\]\(\)\{\}\\\|\_\+\=\`\~])', r' \1 ', raw_token)
                    raw_token = re.sub(' +', ' ', raw_token)
                    tokens += [t for t in raw_token.split(' ') if t]
                else:
                    tokens += [fine_tokens[0]]
            else:
                raise Exception(f'Found no text in raw token {raw_token}')

        return tokens

