# encoding: utf-8
# Created by chenghaomou at 2019-05-01
import os
import subprocess
from collections import defaultdict
from dataclasses import dataclass
from functools import lru_cache
import unidecode as unidecode


@lru_cache(maxsize=64)
def is_ascii(string: str) -> bool: return all(ord(c) < 128 for c in string)


@lru_cache(maxsize=128)
def romanize(string: str, romanization_path: str, language_code: str) -> str:

    cmd = '{}'.format(os.path.join(romanization_path, "bin/uroman.pl"))
    result = subprocess.run([cmd, '--no-cache', '-l', language_code], input=string, stdout=subprocess.PIPE, encoding='utf-8')
    return unidecode.unidecode(result.stdout.strip('\n'))


def load_dictionary(path: str, pos: bool = False) -> dict:
    res = defaultdict(set)
    with open(path) as i:
        for line in i:
            if not pos:
                _, word, gloss = line.strip('\n').split('\t')
            else:
                _, word, _, gloss = line.strip('\n').split('\t')

            res[word.strip(' ')].add(gloss.strip(' '))

    return res


def load_vocabulary(path: str) -> set:
    return set(map(lambda x: x.strip('\n').strip(), open(path).readlines()))


@dataclass
class Romanizer:

    romanization_path: str
    language_code: str

    def run(self, string):

        return romanize(string, self.romanization_path, self.language_code)


def debug(func):
    def wrapper(*args, **kargs):
        result = func(*args, **kargs)
        print(result)
        return result
    return wrapper

